from unittest import TestCase

from viet_text_tools import normalize_diacritics
from viet_text_tools import vietnamese_case_insensitive_sort_key
from viet_text_tools import vietnamese_order_dict
from viet_text_tools import vietnamese_sort_key


class NormalizeDiacriticsTestCase(TestCase):
    def test(self):
        assert normalize_diacritics('hoạ') == 'họa'
        assert normalize_diacritics('choàng') == 'choàng'
        assert normalize_diacritics('thuỷ') == 'thủy'
        assert normalize_diacritics('oà') == 'òa'
        assert normalize_diacritics('toà') == 'tòa'
        assert normalize_diacritics('toàn') == 'toàn'
        assert normalize_diacritics('tòan') == 'toàn'

    def test_new_style(self):
        assert normalize_diacritics('ngòăng', new_style=True) == 'ngoằng'
        assert normalize_diacritics('họa', new_style=True) == 'hoạ'
        assert normalize_diacritics('chòang', new_style=True) == 'choàng'
        assert normalize_diacritics('giừơng', new_style=True) == 'giường'
        assert normalize_diacritics('baỷ', new_style=True) == 'bảy'
        assert normalize_diacritics('cuả', new_style=True) == 'của'
        assert normalize_diacritics('òa', new_style=True) == 'oà'
        assert normalize_diacritics('toàn', new_style=True) == 'toàn'

    def test_decomposed(self):
        word = 'thuỷ'
        expected = 'thủy'
        assert len(word) == 4
        assert len(expected) == 5
        assert normalize_diacritics(word, decomposed=True) == expected


class SortingTestCase(TestCase):
    def test_sort(self):
        words = ['anh', 'ba', 'áo', 'cắt', 'cá', 'cả']
        assert sorted(words) == ['anh', 'ba', 'cá', 'cả', 'cắt', 'áo']
        assert sorted(words, key=vietnamese_sort_key) == ['anh', 'áo', 'ba', 'cả', 'cá', 'cắt']

    def test_curly_quote(self):
        words = ['\u201canh\u201d', 'anh', 'ba', 'áo', 'cắt', 'cá', 'cả']
        assert sorted(words, key=vietnamese_sort_key) == ['anh', 'áo', 'ba', 'cả', 'cá', 'cắt', '\u201canh\u201d']

    def test_chinese_character(self):
        words = ['字', 'anh', 'ba', 'áo', 'cắt', 'cá', 'cả']
        assert sorted(words, key=vietnamese_sort_key) == ['anh', 'áo', 'ba', 'cả', 'cá', 'cắt', '字']

    def test_uppercase_before_lowercase(self):
        words = ['Anh', 'Ba', 'Áo', 'Cắt', 'Cá', 'Cả', 'anh', 'ba', 'áo', 'cắt', 'cá', 'cả']
        assert sorted(words) == ['Anh', 'Ba', 'Cá', 'Cả', 'Cắt', 'anh', 'ba', 'cá', 'cả', 'cắt', 'Áo', 'áo']
        assert sorted(words, key=vietnamese_sort_key) == ['Anh', 'Áo', 'Ba', 'Cả', 'Cá', 'Cắt', 'anh', 'áo', 'ba', 'cả', 'cá', 'cắt']

    def test_case_insensitive(self):
        words = ['Anh', 'Ba', 'Áo', 'Cắt', 'Cá', 'Cả', 'anh', 'ba', 'áo', 'cắt', 'cá', 'cả']
        assert sorted(words) == ['Anh', 'Ba', 'Cá', 'Cả', 'Cắt', 'anh', 'ba', 'cá', 'cả', 'cắt', 'Áo', 'áo']
        assert sorted(words, key=vietnamese_case_insensitive_sort_key) == ['Anh', 'anh', 'Áo', 'áo', 'Ba', 'ba', 'Cả', 'cả', 'Cá', 'cá', 'Cắt', 'cắt']

    def test_numbers_before_letters(self):
        words = ['1', 'anh', 'ba', 'áo', 'cắt', 'cá', 'cả']
        assert sorted(words) == ['1', 'anh', 'ba', 'cá', 'cả', 'cắt', 'áo']
        assert sorted(words, key=vietnamese_sort_key) == ['1', 'anh', 'áo', 'ba', 'cả', 'cá', 'cắt']

    def test_punctuation_before_numbers(self):
        words = ['$', '0', '1', '<', '[', 'anh', 'ba', 'áo', 'cắt', 'cá', 'cả']
        assert sorted(words) == ['$', '0', '1', '<', '[', 'anh', 'ba', 'cá', 'cả', 'cắt', 'áo']
        assert sorted(words, key=vietnamese_sort_key) == ['$', '<', '[', '0', '1', 'anh', 'áo', 'ba', 'cả', 'cá', 'cắt']

    def test_non_vietnamese_characters_go_last(self):
        char = 'Ä'
        assert ord(char) < max(vietnamese_order_dict.values())
        words = ['á', 'ộ', 'ơ', 'ờ', 'Ä']
        assert sorted(words, key=vietnamese_sort_key) == ['á', 'ộ', 'ơ', 'ờ', 'Ä']



import string
import re
import unicodedata
from importlib.metadata import version

__version__ = version(__package__)


def normalize_diacritics(source, new_style=False, decomposed=False):
    tone = r'([\u0300\u0309\u0303\u0301\u0323])'
    combining_breve = r'\u0306'
    combining_circumflex_accent = r'\u0302'
    combining_horn = r'\u031B'
    diacritics = f'{combining_breve}{combining_circumflex_accent}{combining_horn}'
    result = unicodedata.normalize('NFD', source)
    # Put the tone on the second vowel
    result = re.sub(r'(?i){}([aeiouy{}]+)'.format(tone, diacritics), r'\2\1', result)
    # Put the tone on the vowel with a diacritic
    result = re.sub(r'(?i)(?<=[{}])(.){}'.format(diacritics, tone), r'\2\1', result)
    # For vowels that are not oa, oe, uy put the tone on the penultimate vowel
    result = re.sub(r'(?i)(?<=[ae])([iouy]){}'.format(tone), r'\2\1', result)
    result = re.sub(r'(?i)(?<=[oy])([iuy]){}'.format(tone), r'\2\1', result)
    result = re.sub(r'(?i)(?<!q)(u)([aeiou]){}'.format(tone), r'\1\3\2', result)
    result = re.sub(r'(?i)(?<!g)(i)([aeiouy]){}'.format(tone), r'\1\3\2', result)
    if not new_style:
        # Put tone in the symmetrical position
        result = re.sub(r'(?i)(?<!q)([ou])([aeoy]){}(?!\w)'.format(tone), r'\1\3\2', result)
    if decomposed:
        return unicodedata.normalize('NFD', result)
    return unicodedata.normalize('NFC', result)


# Taken from CLDR - Unicode Common Locale Data Repository
# http://demo.icu-project.org/icu-bin/locexp
ACCENTED_CHARACTERS = [
    'a',
    'à',
    'ả',
    'ã',
    'á',
    'ạ',
    'ă',
    'ằ',
    'ẳ',
    'ẵ',
    'ắ',
    'ặ',
    'â',
    'ầ',
    'ẩ',
    'ẫ',
    'ấ',
    'ậ',
    'b',
    'c',
    'd',
    'đ',
    'e',
    'è',
    'ẻ',
    'ẽ',
    'é',
    'ẹ',
    'ê',
    'ề',
    'ể',
    'ễ',
    'ế',
    'ệ',
    'f',
    'g',
    'h',
    'i',
    'ì',
    'ỉ',
    'ĩ',
    'í',
    'ị',
    'j',
    'k',
    'l',
    'm',
    'n',
    'o',
    'ò',
    'ỏ',
    'õ',
    'ó',
    'ọ',
    'ô',
    'ồ',
    'ổ',
    'ỗ',
    'ố',
    'ộ',
    'ơ',
    'ờ',
    'ở',
    'ỡ',
    'ớ',
    'ợ',
    'p',
    'q',
    'r',
    's',
    't',
    'u',
    'ù',
    'ủ',
    'ũ',
    'ú',
    'ụ',
    'ư',
    'ừ',
    'ử',
    'ữ',
    'ứ',
    'ự',
    'v',
    'w',
    'x',
    'y',
    'ỳ',
    'ỷ',
    'ỹ',
    'ý',
    'ỵ',
    'z',
]
VIETNAMESE_CHARACTERS = list(string.punctuation) + list(string.digits) + [x.upper() for x in ACCENTED_CHARACTERS] + ACCENTED_CHARACTERS


class VietnameseOrderDict(dict):
    def __init__(self):
        for i, char in enumerate(VIETNAMESE_CHARACTERS):
            self[char] = i

    def __getitem__(self, k):
        # Any characters that is not in the dictinary has its index shifted out by the number of keys in the dictionary
        # to preserve relative order and give precedence to characters used in Vietnamese
        if k not in self:
            return ord(k) + len(self)
        return super().__getitem__(k)


vietnamese_order_dict = VietnameseOrderDict()


def vietnamese_sort_key(word):
    word = unicodedata.normalize('NFC', word)
    return [vietnamese_order_dict[c] for c in word]


def vietnamese_case_insensitive_sort_key(word):
    word = unicodedata.normalize('NFC', word)
    return [vietnamese_order_dict[c] for c in word.lower()]



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['viet_text_tools', 'viet_text_tools.tests']

package_data = \
{'': ['*']}

setup_kwargs = {
    'name': 'viet-text-tools',
    'version': '0.1.5',
    'description': 'Functions for working with Vietnamese text',
    'long_description': "Viet Text Tools\n===============\n\nFunctions for working with Vietnamese text\n\nInstallation\n------------\n\nTo get the latest stable release from PyPi\n\n.. code-block:: bash\n\n    pip install viet_text_tools\n\nUsage\n-----\n\nnormalize_diacritics()\n~~~~~~~~~~~~~~~~~~~~~~\n\nYou can normalize diacritics for a Vietnamese word.  The return value is in composed (NFC) form\n\n.. code-block:: python\n\n    normalize_diacritics('nghìên') == 'nghiền'\n\nPass ``new_style=True`` to use new style tone placement\n\n.. code-block:: python\n\n    normalize_diacritics('thủy', new_style=True) == 'thuỷ'\n\nPass ``decomposed=True`` to return a string in decomposed (NFD) form\n\n.. code-block:: python\n\n    len(normalize_diacritics('thủy')) == 4\n    len(normalize_diacritics('thủy', decomposed=True)) == 5\n\nvietnamese_sort_key()\n~~~~~~~~~~~~~~~~~~~~~\n\nA key function for use with sorted() to sort Vietnamese text with the correct collation order\n\n.. code-block:: python\n\n    words = ['anh', 'ba', 'áo', 'cắt', 'cá', 'cả']\n    sorted(words) == ['anh', 'ba', 'cá', 'cả', 'cắt', 'áo']\n    sorted(words, key=vietnamese_sort_key) == ['anh', 'áo', 'ba', 'cả', 'cá', 'cắt']\n\nvietnamese_case_insensitive_sort_key()\n~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~\n\nSame as `vietnamese_sort_key()` but case-insensitive.\n",
    'author': 'Enrico Barzetti',
    'author_email': 'enricobarzetti@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/enricobarzetti/viet_text_tools',
    'packages': packages,
    'package_data': package_data,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)
